@@ -16,12 +16,14 @@ module Main (main) where
 
 import XMonad
 
-import Control.Exception (handle)
 import System.IO
 import System.Info
 import System.Environment
 import System.Posix.Process (executeFile)
 
+import Paths_xmonad (version)
+import Data.Version (showVersion)
+
 #ifdef TESTING
 import qualified Properties
 #endif
@@ -31,18 +33,32 @@ import qualified Properties
 main :: IO ()
 main = do
     args <- getArgs
-    let launch = handle (hPrint stderr) buildLaunch >> xmonad defaultConfig
+    let launch = catchIO buildLaunch >> xmonad defaultConfig
     case args of
         []                    -> launch
         ["--resume", _]       -> launch
-        ["--recompile"]       -> recompile False >> return ()
-        ["--recompile-force"] -> recompile True >> return ()
-        ["--version"]         -> putStrLn "xmonad 0.5"
+        ["--help"]            -> usage
+        ["--recompile"]       -> recompile True >> return ()
+        ["--version"]         -> putStrLn ("xmonad " ++ showVersion version)
 #ifdef TESTING
         ("--run-tests":_)     -> Properties.main
 #endif
         _                     -> fail "unrecognized flags"
 
+usage :: IO ()
+usage = do
+    self <- getProgName
+    putStr . unlines $
+        concat ["Usage: ", self, " [OPTION]"] :
+        "Options:" :
+        "  --help                       Print this message" :
+        "  --version                    Print the version number" :
+        "  --recompile                  Recompile your ~/.xmonad/xmonad.hs" :
+#ifdef TESTING
+        "  --run-tests                  Run the test suite" :
+#endif
+        []
+
 -- | Build "~/.xmonad/xmonad.hs" with ghc, then execute it.  If there are no
 -- errors, this function does not return.  An exception is raised in any of
 -- these cases:
