@@ -112,7 +112,7 @@ import qualified Data.Map  as M (Map,insert,delete,empty)
 -- viewable.  We thus need to track which virtual workspaces are
 -- associated (viewed) on which physical screens.  To keep track of
 -- this, StackSet keeps separate lists of visible but non-focused
--- workspaces, and non-visible workspaces.  
+-- workspaces, and non-visible workspaces.
 
 -- $focus
 --
@@ -122,38 +122,6 @@ import qualified Data.Map  as M (Map,insert,delete,empty)
 -- 'delete'.
 --
 
--- |
--- API changes from xmonad 0.1:
--- StackSet constructor arguments changed. StackSet workspace window screen
---
--- * new,                    -- was: empty
---
--- * view,
---
--- * index,
---
--- * peek,                   -- was: peek\/peekStack
---
--- * focusUp, focusDown,     -- was: rotate
---
--- * swapUp, swapDown
---
--- * focus                   -- was: raiseFocus
---
--- * insertUp,               -- was: insert\/push
---
--- * delete,
---
--- * swapMaster,             -- was: promote\/swap
---
--- * member,
---
--- * shift,
---
--- * lookupWorkspace,        -- was: workspace
---
--- * visibleWorkspaces       -- gone.
---
 ------------------------------------------------------------------------
 -- |
 -- A cursor into a non-empty list of workspaces.
@@ -177,7 +145,7 @@ data Screen i l a sid sd = Screen { workspace :: !(Workspace i l a)
     deriving (Show, Read, Eq)
 
 -- |
--- A workspace is just a tag - its index - and a stack
+-- A workspace is just a tag, a layout, and a stack.
 --
 data Workspace i l a = Workspace  { tag :: !i, layout :: l, stack :: Maybe (Stack a) }
     deriving (Show, Read, Eq)
@@ -242,8 +210,7 @@ new _ _ _ = abort "non-positive argument to StackSet.new"
 
 view :: (Eq s, Eq i) => i -> StackSet i l a s sd -> StackSet i l a s sd
 view i s
-    | not (i `tagMember` s)
-      || i == tag (workspace (current s)) = s  -- out of bounds or current
+    | i == tag (workspace (current s)) = s  -- current
 
     | Just x <- L.find ((i==).tag.workspace) (visible s)
     -- if it is visible, it is just raised
@@ -254,7 +221,7 @@ view i s
     = s { current = (current s) { workspace = x }
         , hidden = workspace (current s) : L.deleteBy (equating tag) x (hidden s) }
 
-    | otherwise = s -- can't happen: all workspaces are either invalid, current, visible, or hidden
+    | otherwise = s -- not a member of the stackset
 
   where equating f = \x y -> f x == f y
 
@@ -333,7 +300,7 @@ integrate (Stack x l r) = reverse l ++ x : r
 integrate' :: Maybe (Stack a) -> [a]
 integrate' = maybe [] integrate
 
--- | 
+-- |
 -- /O(n)/. Turn a list into a possibly empty stack (i.e., a zipper):
 -- the first element of the list is current, and the rest of the list
 -- is down.
@@ -446,7 +413,7 @@ mapLayout f (StackSet v vs hs m) = StackSet (fScreen v) (map fScreen vs) (map fW
     fScreen (Screen ws s sd) = Screen (fWorkspace ws) s sd
     fWorkspace (Workspace t l s) = Workspace t (f l) s
 
--- | /O(n)/. Is a window in the StackSet.
+-- | /O(n)/. Is a window in the StackSet?
 member :: Eq a => a -> StackSet i l a s sd -> Bool
 member a s = isJust (findTag a s)
 
