@@ -2,6 +2,9 @@
 module Properties where
 
 import XMonad.StackSet hiding (filter)
+import XMonad.Layout
+import XMonad.Core hiding (workspaces,trace)
+import XMonad.Operations  ( applyResizeIncHint, applyMaxSizeHint )
 import qualified XMonad.StackSet as S (filter)
 
 import Debug.Trace
@@ -136,10 +139,10 @@ prop_empty_I  (n :: Positive Int) l = forAll (choose (1,fromIntegral n)) $  \m -
         invariant $ new l [0..fromIntegral n-1] ms
 
 prop_view_I (n :: NonNegative Int) (x :: T) =
-    n `tagMember` x ==> invariant $ view (fromIntegral n) x
+    invariant $ view (fromIntegral n) x
 
 prop_greedyView_I (n :: NonNegative Int) (x :: T) =
-    n `tagMember` x ==> invariant $ view (fromIntegral n) x
+    invariant $ greedyView (fromIntegral n) x
 
 prop_focusUp_I (n :: NonNegative Int) (x :: T) =
     invariant $ foldr (const focusUp) x [1..n]
@@ -236,6 +239,13 @@ prop_greedyView_current (x :: T) (n :: NonNegative Int) = i `tagMember` x ==>
   where
     i = fromIntegral n
 
+-- greedyView leaves things unchanged for invalid workspaces
+prop_greedyView_current_id (x :: T) (n :: NonNegative Int) = not (i `tagMember` x) ==>
+    tag (workspace $ current (greedyView i x)) == j
+  where
+    i = fromIntegral n
+    j = tag (workspace (current x))
+
 -- greedyView *only* sets the current workspace, and touches Xinerama.
 -- no workspace contents will be changed.
 prop_greedyView_local  (x :: T) (n :: NonNegative Int) = i `tagMember` x ==>
@@ -348,6 +358,10 @@ prop_focusWindow_local (n :: NonNegative Int) (x::T ) =
                        i = fromIntegral n `mod` length s
                    in hidden_spaces (focusWindow (s !! i) x) == hidden_spaces x
 
+-- On an invalid window, the stackset is unmodified
+prop_focusWindow_identity (n :: Char) (x::T ) =
+    not (n `member` x) ==> focusWindow n x == x
+
 -- ---------------------------------------------------------------------
 -- member/findTag
 
@@ -540,13 +554,19 @@ prop_float_reversible n (x :: T) =
         where
             geom = RationalRect 100 100 100 100
 
--- check rectanges were set
-{-
-prop_float_sets_geometry n (x :: T) =
-    n `member` x ==> let y = float n geom x in M.lookup y (floating x) == Just geom
+prop_float_geometry n (x :: T) =
+    n `member` x ==> let s = float n geom x
+                     in M.lookup n (floating s) == Just geom
         where
             geom = RationalRect 100 100 100 100
--}
+
+prop_float_delete n (x :: T) =
+    n `member` x ==> let s = float n geom x
+                         t = delete n s
+                     in not (n `member` t)
+        where
+            geom = RationalRect 100 100 100 100
+
 
 ------------------------------------------------------------------------
 
@@ -605,9 +625,26 @@ prop_rename1 (x::T) o n = o `tagMember` x && not (n `tagMember` x) ==>
     let y = renameTag o n x
             in n `tagMember` y
 
+-- | 
+-- Ensure that a given set of workspace tags is present by renaming
+-- existing workspaces and\/or creating new hidden workspaces as
+-- necessary.
+--
 prop_ensure (x :: T) l xs = let y = ensureTags l xs x
                                 in and [ n `tagMember` y | n <- xs ]
 
+-- adding a tag should create a new hidden workspace
+prop_ensure_append (x :: T) l n =
+    not (n  `tagMember` x)
+      ==>
+   (hidden y /= hidden x        -- doesn't append, renames
+   &&
+   and [ isNothing (stack z) && layout z == l | z <- hidden y, tag z == n ]
+   )
+  where
+    y  = ensureTags l (n:ts) x
+    ts = [ tag z | z <- workspaces x ]
+
 prop_mapWorkspaceId (x::T) = x == mapWorkspace id x
 
 prop_mapWorkspaceInverse (x::T) = x == mapWorkspace predTag (mapWorkspace succTag x)
@@ -619,17 +656,145 @@ prop_mapLayoutId (x::T) = x == mapLayout id x
 prop_mapLayoutInverse (x::T) = x == mapLayout pred (mapLayout succ x)
 
 ------------------------------------------------------------------------
--- some properties for layouts:
+-- The Tall layout
 
 -- 1 window should always be tiled fullscreen
-{-
 prop_tile_fullscreen rect = tile pct rect 1 1 == [rect]
+    where pct = 1/2
 
 -- multiple windows
 prop_tile_non_overlap rect windows nmaster = noOverlaps (tile pct rect nmaster windows)
   where _ = rect :: Rectangle
+        pct = 3 % 100
 
-pct = 3 % 100
+-- splitting horizontally yields sensible results
+prop_split_hoziontal (NonNegative n) x =
+{-
+    trace (show (rect_x x
+                ,rect_width x
+                ,rect_x x + fromIntegral (rect_width x)
+                ,map rect_x xs))
+          $
+-}
+
+        sum (map rect_width xs) == rect_width x
+     &&
+        all (== rect_height x) (map rect_height xs)
+     &&
+        (map rect_x xs) == (sort $ map rect_x xs)
+
+    where
+        xs = splitHorizontally n x
+
+-- splitting horizontally yields sensible results
+prop_splitVertically (r :: Rational) x =
+
+        rect_x x == rect_x a && rect_x x == rect_x b
+      &&
+        rect_width x == rect_width a && rect_width x == rect_width b
+
+{-
+    trace (show (rect_x x
+                ,rect_width x
+                ,rect_x x + fromIntegral (rect_width x)
+                ,map rect_x xs))
+          $
+-}
+
+    where
+        (a,b) = splitVerticallyBy r x
+
+
+-- pureLayout works.
+prop_purelayout_tall n r1 r2 rect (t :: T) =
+    isJust (peek t) ==>
+        length ts == length (index t)
+      &&
+        noOverlaps (map snd ts)
+      &&
+        description layoot == "Tall"
+    where layoot = Tall n r1 r2
+          st = fromJust . stack . workspace . current $ t
+          ts = pureLayout layoot rect st
+
+-- Test message handling of Tall
+
+-- what happens when we send a Shrink message to Tall
+prop_shrink_tall (NonNegative n) (NonZero (NonNegative delta)) (NonNegative frac) =
+        n == n' && delta == delta' -- these state components are unchanged
+    && frac' <= frac  && (if frac' < frac then frac' == 0 || frac' == frac - delta
+                                          else frac == 0 )
+        -- remaining fraction should shrink
+    where
+         l1                   = Tall n delta frac
+         Just l2@(Tall n' delta' frac') = l1 `pureMessage` (SomeMessage Shrink)
+        --  pureMessage :: layout a -> SomeMessage -> Maybe (layout a)
+
+
+-- what happens when we send a Shrink message to Tall
+prop_expand_tall (NonNegative n)
+                 (NonZero (NonNegative delta))
+                 (NonNegative n1)
+                 (NonZero (NonNegative d1)) =
+
+       n == n'
+    && delta == delta' -- these state components are unchanged
+    && frac' >= frac
+    && (if frac' > frac
+           then frac' == 1 || frac' == frac + delta
+           else frac == 1 )
+
+        -- remaining fraction should shrink
+    where
+         frac                 = min 1 (n1 % d1)
+         l1                   = Tall n delta frac
+         Just l2@(Tall n' delta' frac') = l1 `pureMessage` (SomeMessage Expand)
+        --  pureMessage :: layout a -> SomeMessage -> Maybe (layout a)
+
+-- what happens when we send an IncMaster message to Tall
+prop_incmaster_tall (NonNegative n) (NonZero (NonNegative delta)) (NonNegative frac)
+                    (NonNegative k) =
+       delta == delta'  && frac == frac' && n' == n + k
+    where
+         l1                   = Tall n delta frac
+         Just l2@(Tall n' delta' frac') = l1 `pureMessage` (SomeMessage (IncMasterN k))
+        --  pureMessage :: layout a -> SomeMessage -> Maybe (layout a)
+
+
+
+     --   toMessage LT = SomeMessage Shrink
+     --   toMessage EQ = SomeMessage Expand
+     --   toMessage GT = SomeMessage (IncMasterN 1)
+
+
+------------------------------------------------------------------------
+-- Full layout
+
+-- pureLayout works for Full
+prop_purelayout_full rect (t :: T) =
+    isJust (peek t) ==>
+        length ts == 1        -- only one window to view
+      &&
+        snd (head ts) == rect -- and sets fullscreen
+      &&
+        fst (head ts) == fromJust (peek t) -- and the focused window is shown
+
+    where layoot = Full
+          st = fromJust . stack . workspace . current $ t
+          ts = pureLayout layoot rect st
+
+-- what happens when we send an IncMaster message to Full --- Nothing
+prop_sendmsg_full (NonNegative k) =
+         isNothing (Full `pureMessage` (SomeMessage (IncMasterN k)))
+
+prop_desc_full = description Full == show Full
+
+------------------------------------------------------------------------
+
+prop_desc_mirror n r1 r2 = description (Mirror $! t) == "Mirror Tall"
+    where t = Tall n r1 r2
+
+------------------------------------------------------------------------
 
 noOverlaps []  = True
 noOverlaps [_] = True
@@ -645,7 +810,28 @@ noOverlaps xs  = and [ verts a `notOverlap` verts b
         =  (top1 < bottom2 || top2 < bottom1)
         || (right1 < left2 || right2 < left1)
 
--}
+------------------------------------------------------------------------
+-- Aspect ratios
+
+prop_resize_inc (NonZero (NonNegative inc_w),NonZero (NonNegative inc_h))  b@(w,h) =
+    w' `mod` inc_w == 0 && h' `mod` inc_h == 0
+   where (w',h') = applyResizeIncHint a b
+         a = (inc_w,inc_h)
+
+prop_resize_inc_extra ((NonNegative inc_w))  b@(w,h) =
+     (w,h) == (w',h')
+   where (w',h') = applyResizeIncHint a b
+         a = (-inc_w,0::Dimension)-- inc_h)
+
+prop_resize_max (NonZero (NonNegative inc_w),NonZero (NonNegative inc_h))  b@(w,h) =
+    w' <= inc_w && h' <= inc_h
+   where (w',h') = applyMaxSizeHint a b
+         a = (inc_w,inc_h)
+
+prop_resize_max_extra ((NonNegative inc_w))  b@(w,h) =
+     (w,h) == (w',h')
+   where (w',h') = applyMaxSizeHint a b
+         a = (-inc_w,0::Dimension)-- inc_h)
 
 ------------------------------------------------------------------------
 
@@ -653,7 +839,7 @@ main :: IO ()
 main = do
     args <- fmap (drop 1) getArgs
     let n = if null args then 100 else read (head args)
-    (results, passed) <- liftM unzip $ mapM (\(s,a) -> printf "%-25s: " s >> a n) tests
+    (results, passed) <- liftM unzip $ mapM (\(s,a) -> printf "%-40s: " s >> a n) tests
     printf "Passed %d tests!\n" (sum passed)
     when (not . and $ results) $ fail "Not all tests passed!"
  where
@@ -675,6 +861,7 @@ main = do
 
         ,("greedyView : invariant"    , mytest prop_greedyView_I)
         ,("greedyView sets current"   , mytest prop_greedyView_current)
+        ,("greedyView is safe "   ,   mytest prop_greedyView_current_id)
         ,("greedyView idempotent"     , mytest prop_greedyView_idem)
         ,("greedyView reversible"     , mytest prop_greedyView_reversible)
         ,("greedyView is local"       , mytest prop_greedyView_local)
@@ -704,6 +891,7 @@ main = do
 
         ,("focusWindow is local", mytest prop_focusWindow_local)
         ,("focusWindow works"   , mytest prop_focusWindow_works)
+        ,("focusWindow identity", mytest prop_focusWindow_identity)
 
         ,("findTag"           , mytest prop_findIndex)
         ,("allWindows/member"   , mytest prop_allWindowsMember)
@@ -748,13 +936,17 @@ main = do
         ,("shiftWin fix current" , mytest prop_shift_win_fix_current)
 
         ,("floating is reversible" , mytest prop_float_reversible)
+        ,("floating sets geometry" , mytest prop_float_geometry)
+        ,("floats can be deleted", mytest prop_float_delete)
         ,("screens includes current", mytest prop_screens)
+
         ,("differentiate works", mytest prop_differentiate)
         ,("lookupTagOnScreen", mytest prop_lookup_current)
         ,("lookupTagOnVisbleScreen", mytest prop_lookup_visible)
         ,("screens works",      mytest prop_screens_works)
         ,("renaming works",     mytest prop_rename1)
         ,("ensure works",     mytest prop_ensure)
+        ,("ensure hidden semantics",     mytest prop_ensure_append)
 
         ,("mapWorkspace id", mytest prop_mapWorkspaceId)
         ,("mapWorkspace inverse", mytest prop_mapWorkspaceInverse)
@@ -766,12 +958,31 @@ main = do
         ,("new fails with abort",   mytest prop_new_abort)
         ,("shiftWin identity",      mytest prop_shift_win_indentity)
 
-        -- renaming
+        -- tall layout
 
-{-
         ,("tile 1 window fullsize", mytest prop_tile_fullscreen)
         ,("tiles never overlap",    mytest prop_tile_non_overlap)
--}
+        ,("split hozizontally",     mytest prop_split_hoziontal)
+        ,("split verticalBy",       mytest prop_splitVertically)
+
+        ,("pure layout tall",       mytest prop_purelayout_tall)
+        ,("send shrink    tall",    mytest prop_shrink_tall)
+        ,("send expand    tall",    mytest prop_expand_tall)
+        ,("send incmaster tall",    mytest prop_incmaster_tall)
+
+        -- full layout
+
+        ,("pure layout full",       mytest prop_purelayout_full)
+        ,("send message full",      mytest prop_sendmsg_full)
+        ,("describe full",          mytest prop_desc_full)
+
+        ,("describe mirror",        mytest prop_desc_mirror)
+
+        -- resize hints
+        ,("window hints: inc",      mytest prop_resize_inc)
+        ,("window hints: inc all",  mytest prop_resize_inc_extra)
+        ,("window hints: max",      mytest prop_resize_max)
+        ,("window hints: max all ", mytest prop_resize_max_extra)
 
         ]
 
